# -*- coding: utf-8 -*-

"""
todo: add docstring
"""

from .base_document import BaseArsDocument
from .base_document import T_ARS_DOCUMENT
from .resource_type_document import ResourceTypeDocument
from .resource_document import get_utc_now
from .resource_document import to_human_readable_elapsed
from .resource_document import to_utc_dt
from .resource_document import SIMPLE_DT_FMT
from .resource_document import to_simple_dt_fmt
from .resource_document import to_iso_dt_fmt
from .resource_document import get_none_or_default
from .resource_document import get_description
from .resource_document import get_datetime
from .resource_document import get_datetime_simple_fmt
from .resource_document import get_datetime_iso_fmt
from .resource_document import ResourceDocument
from .resource_document import T_ARS_RESOURCE_DOCUMENT
