# -*- coding: utf-8 -*-

"""
Usage::

    from aws_resource_search.tests.fake_aws.api import FakeAws, guid, envs, rand_env
"""

from .main import FakeAws
from .utils import guid
from .utils import envs
from .utils import rand_env