# -*- coding: utf-8 -*-

"""
A terminal application that enables interactive searches for AWS resources.
It is a mini AWS console in your terminal or shell environment.
"""


from ._version import __version__

__short_description__ = (
    "A terminal application that enables interactive searches for AWS resources. "
    "It is a mini AWS console in your terminal or shell environment."
)
__license__ = "GPL"
__author__ = "Sanhe Hu"
__author_email__ = "husanhe@gmail.com"
__maintainer__ = "Sanhe Hu"
__maintainer_email__ = "husanhe@gmail.com"
__github_username__ = "MacHu-GWU"
