# -*- coding: utf-8 -*-

from .vendor.better_enum import BetterStrEnum


class SearcherEnum(BetterStrEnum):
    cloudformation_stack = "cloudformation-stack"
    codebuild_job_run = "codebuild-job-run"
    codebuild_project = "codebuild-project"
    codecommit_repository = "codecommit-repository"
    codepipeline_pipeline = "codepipeline-pipeline"
    dynamodb_table = "dynamodb-table"
    ec2_instance = "ec2-instance"
    ec2_security_group = "ec2-security-group"
    ec2_subnet = "ec2-subnet"
    ec2_vpc = "ec2-vpc"
    ecr_repository = "ecr-repository"
    ecr_repository_image = "ecr-repository-image"
    ecs_cluster = "ecs-cluster"
    ecs_task_run = "ecs-task-run"
    ecs_task_definition_family = "ecs_task_definition_family"
    glue_crawler = "glue-crawler"
    glue_database = "glue-database"
    glue_database_table = "glue-database-table"
    glue_job = "glue-job"
    glue_job_run = "glue-job-run"
    iam_group = "iam-group"
    iam_policy = "iam-policy"
    iam_role = "iam-role"
    iam_user = "iam-user"
    kms_key_alias = "kms-key-alias"
    lambda_function = "lambda-function"
    lambda_function_alias = "lambda-function-alias"
    lambda_layer = "lambda-layer"
    rds_db_cluster = "rds-db-cluster"
    rds_db_instance = "rds-db-instance"
    s3_bucket = "s3-bucket"
    secretsmanager_secret = "secretsmanager-secret"
    sfn_state_machine = "sfn-state-machine"
    sfn_state_machine_execution = "sfn-state-machine-execution"
    sns_topic = "sns-topic"
    sqs_queue = "sqs-queue"
    ssm_parameter = "ssm-parameter"